package de.hampelratte.yaggui;

import java.io.PrintStream;
import java.net.Socket;
import java.util.HashSet;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import de.hampelratte.yaggui.gui.MainFrame;

public class Communicator {
    private static Logger logger = Logger.getLogger();

    private Socket socket;

    private PrintStream ps;

    private ResponseReader rr;

    private MainFrame gui;

    private int session = 1;

    private Hashtable usedSessionIDs = new Hashtable();
    
    private Hashtable downloadToLocateMap = new Hashtable();
    
    private HashSet locateIDs = new HashSet();

    public Communicator(MainFrame gui) {
        this.gui = gui;
        if (((String) Main.confTable.get("autoconnect")).equals("true")) {
            attach();
        }
    }

    public boolean attach() {
        String host = (String) Main.confTable.get("host");
        int port = Integer.parseInt((String) Main.confTable.get("port"));

        try {
            socket = new Socket(host, port);
            ps = new PrintStream(socket.getOutputStream());
            rr = new ResponseReader(socket.getInputStream(), gui, this);
            rr.start();
            System.gc();
            sendToGiftd("ATTACH");
            sendToGiftd("client(yaggui)");
            sendToGiftd("version(" + Main.VERSION + ")");
            sendToGiftd("profile(" + System.getProperty("user.name") + ");");
            return true;
        } catch (Exception e) {
            logger.error(e);
            JOptionPane
                    .showMessageDialog(
                            gui,
                            "<html>Failed to connect to giftd<br>"
                                    + "Perhaps giftd is not running<br>"
                                    + "Otherwise specify the right host in \"Settings\"</html>",
                            "Error", JOptionPane.ERROR_MESSAGE);
            Main.getGUI().showPanel("settingspanel");
            return false;
        }
    }

    public void detach() {
        sendToGiftd("DETACH;");
        rr.stopThread();
    }

    public void stats() {
        sendToGiftd("STATS;");
    }

    public void getShares() {
        while (usedSessionIDs.containsKey(new Integer(session))) {
            session++;
        }
        usedSessionIDs.put(new Integer(session), "SHARES");
        sendToGiftd("SHARES(" + session + ");");
        session++;
    }

    public int search(String query, String exclude, String realm) {
        String r = "";
        String e = "";
        if (!realm.equals(""))
            r = "realm(" + realm + ")";
        if (!exclude.equals(""))
            e = "exclude(" + exclude + ")";

        while (usedSessionIDs.containsKey(new Integer(session))) {
            session++;
        }
        sendToGiftd("SEARCH(" + session + ")\nquery(" + escapeString(query)
                + ") " + e + " " + r + ";");
        usedSessionIDs.put(new Integer(session), "SEARCH");
        return session++;
    }

    public void cancelSearch(int sessionID) {
        sendToGiftd("SEARCH(" + sessionID + ") action(cancel);");
    }

    public void removeSource(Source src) {
        sendToGiftd("DELSOURCE(" + src.getParent().getSession() + ") url("
                + escapeString(src.getURL()) + ");");
    }
    
    public void removeSession(int id) {
        Integer session = new Integer(id);
        usedSessionIDs.remove(session);
        Object locateSession = downloadToLocateMap.get(session);
        if(locateSession != null) {
            locateIDs.remove(locateSession);
            downloadToLocateMap.remove(session);
        }
    }

    public void addSource(String user, String hash, String size, String url,
            String save) {
        sendToGiftd("ADDSOURCE");
        sendToGiftd("user(" + escapeString(user) + ")");
        sendToGiftd("hash(" + hash + ")");
        sendToGiftd("size(" + size + ")");
        sendToGiftd("url(" + escapeString(url) + ")");
        sendToGiftd("save(" + escapeString(save) + ");");
    }
    
    // TODO: don't send ADDSOURCE, if the source is already known
    public void addNewSource(int ID, String user, String hash, String size, String url,
            String save) {
        if(locateIDs.contains(new Integer(ID))) {
            addSource(user, hash, size, url, save);
        } 
    }

    public void changeState(int session, String state) {
        sendToGiftd("TRANSFER(" + session + ")\naction(" + state + ");");
    }

    public int locate(int ID, String query) {
        while (usedSessionIDs.containsKey(new Integer(session))) {
            session++;
        }
        sendToGiftd("LOCATE(" + session + ")\nquery(" + escapeString(query)
                + ");");
        usedSessionIDs.put(new Integer(session), "LOCATE");
        downloadToLocateMap.put(new Integer(ID), new Integer(session));
        locateIDs.add(new Integer(session));
        return session++;
    }

    public int browse(String user) {
        while (usedSessionIDs.containsKey(new Integer(session))) {
            session++;
        }
        sendToGiftd("BROWSE(" + session + ")\nquery(" + escapeString(user)
                + ");");
        usedSessionIDs.put(new Integer(session), "BROWSE");
        return session++;
    }

    public void addSessionID(int ID, String type) {
        this.usedSessionIDs.put(new Integer(ID), type);
    }

    public String getTypeOfQuery(int ID) {
        return (String) this.usedSessionIDs.get(new Integer(ID));
    }

    public void shutdown() {
        sendToGiftd("QUIT;");
    }

    synchronized private void sendToGiftd(String msg) {
        logger.verbose("--> GIFT: " + msg);
        ps.println(msg);
    }

    private String escapeString(String value) {
        char[] toEscape = { '(', ')', '{', '}', ';' };

        // first escape braces and ;
        StringBuffer sb = new StringBuffer(value);
        for (int j = 0; j < toEscape.length; j++) {
            for (int i = 0; i < sb.length(); i++) {
                if (sb.charAt(i) == toEscape[j]) {
                    sb.insert(i, '\\');
                    i++;
                }
            }
        }

        // escape \ (this has to be separated, because otherwise escaping \
        // would be escaped, too)
        for (int i = 0; i < sb.length(); i++) {
            if (sb.charAt(i) == '\\') {
                char c = sb.charAt(i + 1);
                if (c != '(' && c != ')' && c != '{' && c != '}' && c != ';')
                    sb.insert(i, '\\');
                i++;
            }
        }

        value = sb.toString();
        return value;
    }
}