package de.hampelratte.yaggui;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.LookAndFeel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

import com.jgoodies.plaf.plastic.PlasticLookAndFeel;
import com.jgoodies.plaf.plastic.PlasticTheme;

import de.hampelratte.yaggui.gui.MainFrame;

public class Main {
    private static Logger logger = Logger.getLogger();

    public static String confDir = System.getProperty("user.home")
            + System.getProperty("file.separator") + ".yaggui"
            + System.getProperty("file.separator");

    public static final String VERSION = "0.9.2";

    public static Hashtable confTable = new Hashtable();

    public static MainFrame frame;

    public static Communicator com;

    // icons
    public static ImageIcon arrowUpIcon;

    public static ImageIcon arrowDownIcon;

    public static ImageIcon titleIcon;

    public static ImageIcon searchIcon;

    public static ImageIcon sharedIcon;

    public static ImageIcon transferIcon;

    public static ImageIcon peersIcon;

    public static ImageIcon settingsIcon;

    public static ImageIcon disconnectIcon;

    public static ImageIcon stopIcon;

    public static ImageIcon connectedIcon;

    public static ImageIcon disconnectedIcon;

    public static ImageIcon killIcon;

    public static ImageIcon startIcon;

    Main() {
        readConf();
        //loadIcons();
        loadJarIcons();
        frame = new MainFrame();
        com = new Communicator(frame);
    }

    public static MainFrame getGUI() {
        return frame;
    }

    private void readConf() {
        confTable.put("host", "localhost");
        confTable.put("port", "1213");
        confTable.put("autoconnect", "false");
        confTable.put("cddb-host", "www.freedb.org");
        confTable.put("searchJumpBack", "false");
        confTable.put("laftype", "default");
        confTable.put("laf", "none");
        confTable.put("theme", "none");
        confTable.put("giftd", "giftd");
        try {
            File conf = new File(confDir + "yaggui.conf");
            BufferedReader br = new BufferedReader(new FileReader(conf));

            String line;
            while ((line = br.readLine()) != null) {
                line = line.trim();
                if (!line.equals("") && (line.indexOf("=") != -1)
                        && (line.charAt(0) != '#')) {
                    try {
                        StringTokenizer st = new StringTokenizer(line, "=");
                        confTable.put(st.nextToken().trim(), st.nextToken()
                                .trim());
                    } catch (Exception e) {
                        logger.warn("Invalid parameter in yaggui.conf");
                        logger.debug("\t" + e);
                    }
                }
            }
        } catch (IOException e) {
            logger
                    .info("Couldn't find yaggui.conf, starting with default settings");
        }
    }

    private void loadJarIcons() {
        logger.info("Loading images");
        Class res = this.getClass();
        URL url = res.getResource("icons/arrow-down.gif");
        Main.arrowDownIcon = new ImageIcon(url);
        url = res.getResource("icons/arrow-up.gif");
        Main.arrowUpIcon = new ImageIcon(url);
        //url = res.getResource("icons/logo.gif");
        url = res.getResource("icons/logo.png");
        Main.titleIcon = new ImageIcon(url);
        url = res.getResource("icons/search.gif");
        Main.searchIcon = new ImageIcon(url);
        url = res.getResource("icons/shared.gif");
        Main.sharedIcon = new ImageIcon(url);
        url = res.getResource("icons/transfer.gif");
        Main.transferIcon = new ImageIcon(url);
        url = res.getResource("icons/peers.gif");
        Main.peersIcon = new ImageIcon(url);
        url = res.getResource("icons/settings.gif");
        Main.settingsIcon = new ImageIcon(url);
        url = res.getResource("icons/disconnect.png");
        Main.disconnectIcon = new ImageIcon(url);
        url = res.getResource("icons/stop.gif");
        Main.stopIcon = new ImageIcon(url);
        url = res.getResource("icons/connected.png");
        Main.connectedIcon = new ImageIcon(url);
        url = res.getResource("icons/disconnected.png");
        Main.disconnectedIcon = new ImageIcon(url);
        url = res.getResource("icons/kill.png");
        Main.killIcon = new ImageIcon(url);
        url = res.getResource("icons/startgift.png");
        Main.startIcon = new ImageIcon(url);
    }

    public static ImageIcon loadIcon(String path) {
        Class res = Main.class;
        URL url = res.getResource(path);
        return new ImageIcon(url);
    }

    public static void writeConf() {
        try {
            File dir = new File(confDir);
            if (!dir.exists()) {
                dir.mkdir();
            }
            File conf = new File(confDir + "yaggui.conf");
            if (!conf.exists()) {
                conf.createNewFile();
            }
            BufferedReader br = new BufferedReader(new FileReader(conf));
            String line;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                line = line.trim();

                if (!line.equals("") && line.charAt(0) != '#'
                        && line.indexOf("=") != -1) {
                    StringTokenizer st = new StringTokenizer(line, "=");
                    String key = st.nextToken().trim();
                    sb.append(key);
                    sb.append("=");
                    sb.append(confTable.get(key));
                    sb.append(System.getProperty("line.separator"));
                    confTable.remove(key);
                } else {
                    sb.append(line);
                    sb.append(System.getProperty("line.separator"));
                }
            }

            br.close();

            PrintWriter pw = new PrintWriter(new FileOutputStream(conf));
            pw.write(sb.toString());
            Enumeration enumeration = confTable.keys();
            while (enumeration.hasMoreElements()) {
                String key = (String) enumeration.nextElement();
                Object value = confTable.get(key);
                pw.println(key + "=" + value);
            }
            pw.close();
            logger.info("Writing yaggui.conf:..done");
        } catch (Exception e) {
            logger.error("Writing yaggui.conf:..done " + e);
        }
    }

    public static void main(String[] args) {
        // write output to a file
        /*
         * try { java.io.PrintStream ps = new java.io.PrintStream(new
         * java.io.FileOutputStream("yaggui_log.txt")); System.setOut(ps);
         * System.setErr(ps); } catch(Exception e) {
         * e.printStackTrace(System.err); }
         */

        // it's a little bit ugly to set the laf within the main,
        // but it seems to be the only way
        // to set it without any errors or strange behaviour
        //MetalTheme theme = new LightBlueTheme();
        //MetalLookAndFeel.setCurrentTheme(theme);
        de.hampelratte.yaggui.gui.SplashScreen splash = new de.hampelratte.yaggui.gui.SplashScreen();

        UIDefaults table = UIManager.getDefaults();
        Font sansSerif = new Font("SansSerif", Font.PLAIN, 11);

        table.put("Label.font", sansSerif);
        table.put("SplitPane.border", BorderFactory.createEmptyBorder(0, 0, 0,
                0));
        table.put("Menu.font", sansSerif);
        table.put("MenuItem.font", sansSerif);
        table.put("CheckBox.font", sansSerif);
        table.put("PasswordField.font", sansSerif);
        table.put("TextField.font", sansSerif);
        table.put("TextPane.font", sansSerif);
        table.put("TextArea.font", sansSerif);
        table.put("Button.font", sansSerif);
        table.put("RadioButton.font", sansSerif);
        table.put("List.font", sansSerif);
        table.put("Statusbar.background", new Color(230, 230, 230));

        String lafString = "none";
        String themeString = "none";
        try {
            File conf = new File(confDir + "yaggui.conf");
            BufferedReader br = new BufferedReader(new FileReader(conf));

            String line;
            while ((line = br.readLine()) != null) {
                line = line.trim();
                if (!line.equals("") && (line.indexOf("=") != -1)
                        && (line.charAt(0) != '#')) {
                    try {
                        StringTokenizer st = new StringTokenizer(line, "=");
                        String key = st.nextToken();
                        String value = st.nextToken();

                        if (key.toLowerCase().equals("laf"))
                            lafString = value;
                        else if (key.toLowerCase().equals("theme"))
                            themeString = value;
                    } catch (Exception e) {
                        logger.error("Invalid parameter in yaggui.conf " + e);
                    }
                }
            }

            if (!(themeString.equals("null") || themeString.equals("none")
                    || lafString.equals("null") || lafString.equals("none"))) {
                Class c = Class.forName("com.jgoodies.plaf.plastic.theme."
                        + themeString);
                PlasticTheme ptheme = (PlasticTheme) c.newInstance();
                PlasticLookAndFeel.setMyCurrentTheme(ptheme);
            }

            if (!(lafString.equals("null") || lafString.equals("none"))) {
                Class c = Class.forName("com.jgoodies.plaf.plastic."
                        + lafString);
                LookAndFeel lookandfeel = (LookAndFeel) c.newInstance();
                UIManager.setLookAndFeel(lookandfeel);
                table.put("Statusbar.background", UIManager
                        .get("Panel.background"));
            }

            br.close();
        } catch (Exception e) {
            logger
                    .warn("Couldn't load laf from yaggui.conf, starting with default laf\n\tReason: "
                            + e);
        }

        new Main();
        splash.dispose();
    }
}