package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import de.hampelratte.yaggui.Command;
import de.hampelratte.yaggui.Load;
import de.hampelratte.yaggui.Logger;
import de.hampelratte.yaggui.Main;
import de.hampelratte.yaggui.Source;
import de.hampelratte.yaggui.Subcommand;
import de.hampelratte.yaggui.gui.treetable.TreeTable;
import de.hampelratte.yaggui.gui.treetable.TreeTableCellRenderer;
import de.hampelratte.yaggui.gui.treetable.TreeTableModel;

class DownloadPanel extends JPanel implements ActionListener {
    private Logger logger = Logger.getLogger();

    private TreeTable table;

    private TreeTableModel model;

    private JPopupMenu popupDown = new JPopupMenu();

    private JPopupMenu popupSrc = new JPopupMenu();

    private JMenuItem popSrcCancel = new JMenuItem("Remove this Source");

    private JMenuItem popSrcBrowse = new JMenuItem("Browse files");

    private JMenuItem popCancel = new JMenuItem("Cancel");

    private JMenuItem popPause = new JMenuItem("Pause");

    private JMenuItem popResume = new JMenuItem("Resume");

    private JMenuItem popSearch = new JMenuItem("Find more sources");

    DownloadPanel() {
        setLayout(new BorderLayout());

        table = new TreeTable();
        model = new TreeTableModel(table, 7);
        table.setModel(model);
        table.getColumnModel().getColumn(0).setHeaderValue("File");
        table.getColumnModel().getColumn(1).setHeaderValue("User");
        table.getColumnModel().getColumn(2).setHeaderValue("Progress");
        table.getColumnModel().getColumn(3).setHeaderValue("State");
        table.getColumnModel().getColumn(4).setHeaderValue("Rate");
        table.getColumnModel().getColumn(5).setHeaderValue("Time remaining");
        table.getColumnModel().getColumn(6).setHeaderValue("Transfered/Total");
        for (int i = 0; i < table.getColumnCount(); i++) {
            TableColumn col = table.getColumnModel().getColumn(i);
            col.setCellRenderer(new TreeTableCellRenderer());
        }
        table.setShowGrid(false);
        table.setRowHeight(20);
        table.setRowMargin(0);
        table.setShowVerticalLines(false);
        table.setShowHorizontalLines(false);

        for (int i = model.getColumnCount() - 1; i >= 0; i--) {
            DefaultTableColumnModel tcm = (DefaultTableColumnModel) table
                    .getColumnModel();

            TableColumn tc = tcm.getColumn(i);
            if (i == 2) {
                tc.setCellRenderer(new ProgressCellRenderer());
            }
        }

        int[] columnWidth = { 300, 100, 100, 100, 100, 130, 200 };
        TableColumnModel tcmodel = table.getColumnModel();
        int n = columnWidth.length;
        for (int i = 0; i < n; i++) {
            tcmodel.getColumn(i).setPreferredWidth(columnWidth[i]);
        }

        JScrollPane scrollpane = new JScrollPane(table);
        scrollpane.setBorder(BorderFactory.createEmptyBorder());

        JLabel dirLabel = new JLabel("<html><b>Downloads</b></html>");
        //dirLabel.setBorder(BorderFactory.createEtchedBorder());
        add(dirLabel, BorderLayout.NORTH);
        add(scrollpane, BorderLayout.CENTER);

        popCancel.addActionListener(this);
        popPause.addActionListener(this);
        popResume.addActionListener(this);
        popSearch.addActionListener(this);
        popupDown.add(popResume);
        popupDown.add(popPause);
        popupDown.add(popCancel);
        popupDown.addSeparator();
        popupDown.add(popSearch);

        popSrcCancel.addActionListener(this);
        popSrcBrowse.addActionListener(this);
        popupSrc.add(popSrcBrowse);
        popupSrc.add(popSrcCancel);

        table.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isShiftDown() || e.isControlDown() || e.isAltDown()) {
                    return;
                } else if (e.isMetaDown()) {
                    int row = table.rowAtPoint(e.getPoint());
                    Vector r = model.getRowAt(row);
                    Object lastSelectedItem = r.firstElement();
                    if (lastSelectedItem instanceof Load) {
                        popupDown.show(e.getComponent(), e.getX(), e.getY());
                    } else if (lastSelectedItem instanceof Source) {
                        popupSrc.show(e.getComponent(), e.getX(), e.getY());
                    }
                }
            }
        });
    }

    public void addLoad(Command cmd, int session) {
        model.addLoad(createLoadFromCmd(cmd, session));
    }

    public void addSource(Command cmd, int session) {
        Source src = this.createSource(cmd);
        model.addSource(src, session);
        revalidate();
    }

    public void delSource(Command cmd, int session) {
        Source src = this.createSource(cmd);
        model.removeSource(src, session);
        revalidate();
        repaint();
    }

    private Source createSource(Command cmd) {
        Source src = new Source();
        String user = (String) cmd.get("user");
        String url = (String) cmd.get("url");
        String protostate = (String) cmd.get("status");
        String status = (String) cmd.get("statusgrl");
        String start = (String) cmd.get("start");
        String total = (String) cmd.get("total");
        String transmit = (String) cmd.get("transmit");

        if (user == null)
            user = "";
        if (url == null)
            url = "";
        if (protostate == null)
            protostate = "";
        if (status == null)
            status = "";
        if (start == null)
            start = "0";
        if (total == null)
            total = "0";
        if (transmit == null)
            transmit = "0";

        src.setUser(user);
        src.setURL(url);
        src.setProtocolState(protostate);
        src.setStart(Long.parseLong(start));
        src.setTotal(Long.parseLong(total));
        src.setTransmitted(Long.parseLong(transmit));
        src.setThroughput(0);
        src.setState(status);

        return src;
    }

    private Load createLoadFromCmd(Command cmd, int session) {
        Load l = new Load(session);

        l.setFile((String) cmd.get("file"));
        l.setHash((String) cmd.get("hash"));
        l.setState((String) cmd.get("state"));
        l.setSize(Long.parseLong((String) cmd.get("size")));
        l.setTransmitted(Long.parseLong((String) cmd.get("transmit")));
        l.setShared(Boolean.getBoolean((String) cmd.get("shared")));

        String through = (String) cmd.get("throughput");
        String elapsed = (String) cmd.get("elapsed");
        if (through == null)
            through = "0";
        if (elapsed == null)
            elapsed = "0";

        l.setElapsed(Integer.parseInt(elapsed));
        l.setThroughput(Integer.parseInt(through));

        Enumeration enumeration = cmd.getSubCmds().elements();
        while (enumeration.hasMoreElements()) {
            Source src = new Source(l);
            Subcommand subcmd = (Subcommand) enumeration.nextElement();

            String user = (String) subcmd.get("user");
            String url = (String) subcmd.get("url");
            String protostate = (String) subcmd.get("status");
            String status = (String) subcmd.get("statusgrl");
            String start = (String) subcmd.get("start");
            String total = (String) subcmd.get("total");
            String transmit = (String) subcmd.get("transmit");

            if (user == null)
                user = "";
            if (url == null)
                url = "";
            if (protostate == null)
                protostate = "";
            if (status == null)
                status = "";
            if (start == null)
                start = "0";
            if (total == null)
                total = "0";
            if (transmit == null)
                transmit = "0";

            // calculate throughput
            int throughput = 0;
            try {
                Load equalOne = null;
                Enumeration enumeration2 = model.getLoads().elements();
                while (enumeration2.hasMoreElements()) {
                    Load current = (Load) enumeration2.nextElement();
                    if (current.equals(l)) {
                        equalOne = current;
                    }
                }

                for (int i = 0; i < equalOne.getSourceCount(); i++) {
                    Source current = (Source) equalOne.getSource(i);
                    if (current.getUser().equals(user)) {
                        throughput = (int) (Long.parseLong(transmit) - current
                                .getTransmitted());
                    }
                }
            } catch (Exception e) {
            }

            src.setUser(user);
            src.setURL(url);
            src.setProtocolState(protostate);
            src.setStart(Long.parseLong(start));
            src.setTotal(Long.parseLong(total));
            src.setTransmitted(Long.parseLong(transmit));
            src.setThroughput(throughput);
            src.setState(status);

            l.addSource(src);
        }
        return l;
    }

    public void changeLoad(Command cmd, int session) {
        Load l = this.createLoadFromCmd(cmd, session);
        model.updateLoad(l);
        revalidate();
        repaint();
    }

    public void delLoad(int session) {
        Main.com.removeSession(session);
        model.removeLoad(session);
        revalidate();
        repaint();
    }

    public void actionPerformed(ActionEvent e) {
        int rows[] = table.getSelectedRows();
        for (int i = 0; i < rows.length; i++) {
            Object o = table.getModel().getValueAt(rows[i], 0);

            if (e.getSource() == popCancel) {
                if (o instanceof Load) {
                    Load load = (Load) o;
                    int session = load.getSession();
                    Main.com.changeState(session, "cancel");
                }
            } else if (e.getSource() == popPause) {
                if (o instanceof Load) {
                    Load load = (Load) o;
                    int session = load.getSession();
                    Main.com.changeState(session, "pause");
                }
            } else if (e.getSource() == popResume) {
                if (o instanceof Load) {
                    Load load = (Load) o;
                    int session = load.getSession();
                    Main.com.changeState(session, "unpause");
                }
            } else if (e.getSource() == popSearch) {
                if (o instanceof Load) {
                    Load load = (Load) o;
                    int session = load.getSession();
                    Main.com.locate(session, load.getHash());
                }
            } else if (e.getSource() == popSrcCancel) {
                if (o instanceof Source) {
                    Source src = (Source)o;
                    Main.com.removeSource(src);
                }
            } else if (e.getSource() == popSrcBrowse) {
                if (o instanceof Source) {
                    Source src = (Source)o;
	                Main.getGUI().getSearchPanel().addBrowseTab(src);
	                Main.getGUI().showPanel("searchpanel");
                }
            }
        }
    }

    public void resetTable() {
        model.removeAll();
        table.repaint();
    }
}