package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSplitPane;
import javax.swing.JTextField;

import de.hampelratte.yaggui.Command;
import de.hampelratte.yaggui.Main;
import de.hampelratte.yaggui.Result;
import de.hampelratte.yaggui.Source;
import de.hampelratte.yaggui.cddb.AlbumGetter;
import de.hampelratte.yaggui.gui.cddb.CDDBPanel;

public class SearchPanel extends AbstractPanel implements ActionListener {
    private JPanel leftPanel = new JPanel(new GridLayout(15, 1));

    private JLabel searchLabel = new JLabel("Search for: ");

    private JTextField searchField = new JTextField("");

    private JTextField excludeField = new JTextField("");

    private JSplitPane splitpane = new JSplitPane();

    private CloseableTabbedPane resultsPane = new CloseableTabbedPane();

    private MainFrame gui;

    private Hashtable sessionComponentMap = new Hashtable();

    private ButtonGroup bg = new ButtonGroup();

    private JRadioButton radioEverything = new JRadioButton("Everything");

    private JRadioButton radioAudio = new JRadioButton("Audio");

    private JRadioButton radioVideo = new JRadioButton("Video");

    private JRadioButton radioImages = new JRadioButton("Images");

    private JRadioButton radioText = new JRadioButton("Text");

    private JRadioButton radioDocuments = new JRadioButton("Documents");

    private JRadioButton radioSoftware = new JRadioButton("Software");

    private JRadioButton radioCDDB = new JRadioButton("\"CDDB Download\"");

    private JCheckBox jumpBack = new JCheckBox(
            "<html>Always jump back<br>to standard search</html>");

    public SearchPanel(JPanel parent, MainFrame gui) {
        super(parent);
        this.gui = gui;
        setBorder("Search for files");

        jumpBack.addActionListener(this);
        if (Main.confTable.get("searchJumpBack").equals("true"))
            jumpBack.setSelected(true);

        splitpane.setDividerSize(8);
        splitpane.setDividerLocation(200);
        splitpane.setOneTouchExpandable(true);
        splitpane.setLeftComponent(new JPanel());
        splitpane.setEnabled(false);

        searchField.setPreferredSize(new Dimension(150, 25));
        searchField.addActionListener(this);
        excludeField.setPreferredSize(new Dimension(150, 25));
        excludeField.addActionListener(this);

        radioEverything.setSelected(true);
        bg.add(radioEverything);
        bg.add(radioAudio);
        bg.add(radioVideo);
        bg.add(radioImages);
        bg.add(radioText);
        bg.add(radioDocuments);
        bg.add(radioSoftware);
        bg.add(radioCDDB);

        leftPanel.add(new JLabel("Search for: "));
        leftPanel.add(searchField);
        leftPanel.add(new JLabel("Exclude: "));
        leftPanel.add(excludeField);
        leftPanel.add(new JLabel(" "));
        leftPanel.add(radioEverything);
        leftPanel.add(radioAudio);
        leftPanel.add(radioVideo);
        leftPanel.add(radioImages);
        leftPanel.add(radioText);
        leftPanel.add(radioDocuments);
        leftPanel.add(radioSoftware);
        leftPanel.add(radioCDDB);
        leftPanel.add(new JLabel(" "));
        leftPanel.add(jumpBack);

        leftPanel.setPreferredSize(new Dimension(180, 350));

        splitpane.setRightComponent(resultsPane);
        JPanel dummy = new JPanel();
        dummy.add(leftPanel);
        dummy.setBorder(BorderFactory.createEtchedBorder());
        splitpane.setLeftComponent(dummy);
        add(splitpane, BorderLayout.CENTER);
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == searchField || e.getSource() == excludeField) {
            if (!gui.connected) {
                JOptionPane.showMessageDialog(parent,
                        "yaggui is not connected to giftd", "Error",
                        JOptionPane.ERROR_MESSAGE);
            } else {
                String query = searchField.getText();
                String exclude = excludeField.getText();
                String realm = "everything";

                if (!this.radioCDDB.isSelected()) {
                    if (this.radioAudio.isSelected())
                        realm = "audio";
                    else if (this.radioDocuments.isSelected())
                        realm = "documents";
                    else if (this.radioImages.isSelected())
                        realm = "images";
                    else if (this.radioSoftware.isSelected())
                        realm = "software";
                    else if (this.radioText.isSelected())
                        realm = "text";
                    else if (this.radioVideo.isSelected())
                        realm = "video";

                    int sessionID = Main.com.search(query, exclude, realm);

                    // create ResultPanel
                    String type = "";
                    if (realm.equals("everything"))
                        type = "";
                    else
                        type = " (" + realm + ")";
                    ResultPanel rp = new ResultPanel(sessionID, query + type);
                    resultsPane.addTab(query + type, rp);
                    this.sessionComponentMap.put(new Integer(sessionID), rp);
                } else {
                    CDDBPanel cddb = new CDDBPanel(this);
                    resultsPane.addTab(query + " (cddb)", cddb);
                    new AlbumGetter(searchField.getText(), cddb).start();
                }

                resultsPane.setSelectedIndex(resultsPane.getTabCount() - 1);
                // reset searchPanel
                if (jumpBack.isSelected()) {
                    radioEverything.setSelected(true);
                    excludeField.setText("");
                }
                searchField.setSelectionStart(0);
                searchField.setSelectionEnd(query.length());
            }
        } else if (e.getSource() == jumpBack) {
            Main.confTable.put("searchJumpBack", new Boolean(jumpBack
                    .isSelected()).toString());
        }
    }

    public void addResult(Hashtable cmd) {
        String line = (String) cmd.get("command");
        int start = line.indexOf("(") + 1;
        int stop = line.indexOf(")");
        int ID = Integer.parseInt(line.substring(start, stop));
        Integer sessionID = new Integer(ID);
        ResultPanel rp = (ResultPanel) this.sessionComponentMap.get(sessionID);
        int hits = rp.addResult(new Result((Command) cmd));

        /*
         * The following block shows the result count in the tab. but the
         * repaint of the tab is so slow, that the whole search slows down :-(
         * TODO: Maybe a better repaint, or repaint only in small intervals
         * 200ms e.g.
         */
        /*
         * String title = rp.getTitle(); hits--; String oldtitle = ""; if(hits
         * !=0) oldtitle = title + " (" + (hits) + ")"; else oldtitle = title;
         * title += " (" + (++hits) + ")"; int tabIndex =
         * resultsPane.indexOfTab(oldtitle); if(!rp.getEnded())
         * resultsPane.setTitleAt(tabIndex, title);
         */
    }

    public void setEnded(Integer sessionID) {
        ResultPanel rp = (ResultPanel) this.sessionComponentMap.get(sessionID);
        rp.setEnded();
    }

    public void setAllEnded() {
        Enumeration enumeration = this.sessionComponentMap.keys();
        while (enumeration.hasMoreElements()) {
            Object key = enumeration.nextElement();
            ResultPanel rp = (ResultPanel) this.sessionComponentMap.get(key);
            rp.setEnded();
        }
    }

    public void addCDDBSearchTab(String query) {
        int sessionID = Main.com.search(searchField.getText() + " " + query,
                "", "audio");
        String title = searchField.getText() + " " + query + " (cddb-title)";
        ResultPanel rp = new ResultPanel(sessionID, title);
        resultsPane.addTab(title, rp);
        this.sessionComponentMap.put(new Integer(sessionID), rp);
    }

    public void addBrowseTab(Source src) {
        int sessionID = Main.com.browse(src.getUser());
        String title = src.getUserName() + " (user)";
        ResultPanel rp = new ResultPanel(sessionID, title);
        resultsPane.addTab(title, rp);
        this.sessionComponentMap.put(new Integer(sessionID), rp);
    }
}