package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

import com.jgoodies.plaf.plastic.PlasticLookAndFeel;
import com.jgoodies.plaf.plastic.PlasticTheme;

import de.hampelratte.yaggui.Logger;
import de.hampelratte.yaggui.Main;

class SettingsPanel extends AbstractPanel implements ActionListener,
        TreeSelectionListener {

    private static Logger logger = Logger.getLogger();

    // main panel
    private JSplitPane splitpane = new JSplitPane();

    private DefaultMutableTreeNode root = null;

    private JTree naviTree;

    private JPanel mainPanel = new JPanel();

    private CardLayout cards = new CardLayout();

    // connection panel
    private DefaultMutableTreeNode connectionNode = null;

    private JPanel panel = new JPanel();

    private JTextField host = new JTextField();

    private JTextField port = new JTextField();

    private JTextField cddbhost = new JTextField();

    private JCheckBox autoconnect = new JCheckBox("");

    // laf panel
    private DefaultMutableTreeNode lafNode = null;

    private JPanel lafPanel = new JPanel();

    private String[] lafs = { "Default", "Plastic", "Plastic 3D", "Plastic XP" };

    private JComboBox lafCombo = new JComboBox(lafs);

    private String[] themes = { "BrownSugar", "DarkStar", "DesertBlue",
            "DesertBluer", "DesertGreen", "DesertRed", "DesertYellow",
            "ExperienceBlue", "ExperienceGreen", "InvertedColorTheme",
            "Silver", "SkyBlue", "SkyBluer", "SkyBluerTahoma", "SkyGreen",
            "SkyKrupp", "SkyPink", "SkyRed", "SkyYellow" };

    private JComboBox themeCombo = new JComboBox(themes);

    // giftd panel
    private DefaultMutableTreeNode giftdNode = null;

    private JPanel giftdPanel = new JPanel();

    private JTextField giftd = new JTextField(30);

    private JButton browseButton = new JButton("Browse");

    // control panel to the south
    private JButton apply = new JButton("Apply Settings");

    private JPanel controlPanel = new JPanel();

    SettingsPanel(JPanel parent) {
        super(parent);
        initGUI();
    }

    private void initGUI() {
        // panel on the left side of the splitpane
        mainPanel.setLayout(cards);

        // naviTree
        root = new DefaultMutableTreeNode("Settings");
        connectionNode = new DefaultMutableTreeNode("Connection");
        root.add(connectionNode);
        lafNode = new DefaultMutableTreeNode("Look and Feel");
        root.add(lafNode);
        giftdNode = new DefaultMutableTreeNode("giFTd");
        root.add(giftdNode);

        naviTree = new JTree(root);
        naviTree.treeDidChange();
        naviTree.setShowsRootHandles(true);
        naviTree.setRootVisible(false);
        naviTree.getSelectionModel().addTreeSelectionListener(this);

        // splitpane
        splitpane.setLeftComponent(new JScrollPane(naviTree));
        splitpane.setRightComponent(mainPanel);
        splitpane.setDividerSize(5);
        splitpane.setDividerLocation(200);

        // control panel to the south
        apply.addActionListener(this);
        controlPanel.add(apply);

        // connection panel
        //                 Gridlayout(rows, cols)
        panel.setLayout(null);
        JLabel lHost = new JLabel("Host");
        lHost.setBounds(20, 30, 130, 25);
        host.setBounds(160, 30, 200, 25);
        panel.add(lHost);
        panel.add(host);
        JLabel lPort = new JLabel("Port");
        lPort.setBounds(20, 70, 130, 25);
        port.setBounds(160, 70, 200, 25);
        panel.add(lPort);
        panel.add(port);
        JLabel lFreedb = new JLabel("Freedb hostname");
        lFreedb.setBounds(20, 110, 130, 25);
        cddbhost.setBounds(160, 110, 200, 25);
        panel.add(lFreedb);
        panel.add(cddbhost);
        JLabel lAutoconnect = new JLabel("Autoconnect on startup");
        lAutoconnect.setBounds(20, 150, 130, 25);
        autoconnect.setBounds(160, 150, 200, 25);
        panel.add(lAutoconnect);
        panel.add(autoconnect);
        panel.setBorder(BorderFactory.createTitledBorder("Connection"));
        host.setText((String) Main.confTable.get("host"));
        port.setText((String) Main.confTable.get("port"));
        cddbhost.setText((String) Main.confTable.get("cddb-host"));
        if (((String) Main.confTable.get("autoconnect")).equals("true"))
            autoconnect.setSelected(true);
        mainPanel.add(panel, "CONNECTION");

        // lafPanel
        lafPanel.setPreferredSize(new Dimension(200, 200));
        lafPanel.setLayout(null);
        lafCombo.setBounds(20, 30, 160, 25);
        lafPanel.add(lafCombo);
        themeCombo.setBounds(20, 65, 160, 25);
        lafPanel.add(themeCombo);
        JLabel note = new JLabel("<html>Note: the new look and feel<br>"
                + "will be displayed correctly<br>"
                + "only after a restart of YAGGUI!<br>"
                + "I recommend a restart.</html>");
        note.setBounds(20, 110, 160, 75);
        lafPanel.add(note);
        lafPanel.setBorder(BorderFactory.createTitledBorder("Look And Feel"));
        mainPanel.add(lafPanel, "LAF");
        String laf = (String) Main.confTable.get("laf");
        String theme = (String) Main.confTable.get("theme");
        if (laf.equals("null"))
            themeCombo.setEnabled(false);
        else if (laf.equals("PlasticLookAndFeel"))
            lafCombo.setSelectedItem("Plastic");
        else if (laf.equals("Plastic3DLookAndFeel"))
            lafCombo.setSelectedItem("Plastic 3D");
        else if (laf.equals("PlasticXPLookAndFeel"))
            lafCombo.setSelectedItem("Plastic XP");
        else
            themeCombo.setEnabled(false);
        try {
            themeCombo.setSelectedItem(theme);
        } catch (Exception e) {
        }

        // giftd panel
        giftdPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        giftdPanel.setPreferredSize(new Dimension(500, 100));
        giftdPanel.setBorder(BorderFactory.createTitledBorder("giFTd"));
        JLabel label = new JLabel("Path to giftd");
        label.setBounds(10, 10, 200, 25);
        giftdPanel.add(label);
        giftd.setBounds(220, 10, 200, 25);
        giftd.setText((String) Main.confTable.get("giftd"));
        giftdPanel.add(giftd);
        browseButton.setBounds(430, 10, 100, 25);
        giftdPanel.add(browseButton);
        mainPanel.add(giftdPanel, "GIFTD");

        // add listeners to the components
        apply.addActionListener(this);
        lafCombo.addActionListener(this);
        themeCombo.addActionListener(this);
        browseButton.addActionListener(this);

        // the dialog itself
        cards.show(mainPanel, "CONNECTION");
        setBorder(BorderFactory.createTitledBorder("Settings"));
        setLayout(new BorderLayout());
        add(splitpane, BorderLayout.CENTER);
        add(controlPanel, BorderLayout.SOUTH);
    }

    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode source = (DefaultMutableTreeNode) e.getPath()
                .getLastPathComponent();

        if (source == giftdNode) {
            cards.show(mainPanel, "GIFTD");
        } else if (source == lafNode) {
            cards.show(mainPanel, "LAF");
        } else if (source == connectionNode) {
            cards.show(mainPanel, "CONNECTION");
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == apply) {
            applyConfig();
        } else if (e.getSource() == lafCombo) {
            String laf = (String) lafCombo.getSelectedItem();
            if (laf.indexOf("Plastic") != -1)
                themeCombo.setEnabled(true);
            else
                themeCombo.setEnabled(false);
        } else if (e.getSource() == browseButton) {
            JFileChooser fc = new JFileChooser();
            int returnVal = fc.showOpenDialog(parent);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                giftd.setText(file.toString());
            }
        }
    }

    private void applyConfig() {
        try {
            Main.confTable.put("host", host.getText());
            Main.confTable.put("port", port.getText());
            Main.confTable.put("cddb-host", cddbhost.getText());
            Main.confTable.put("giftd", giftd.getText());
            String state = "false";
            if (autoconnect.isSelected())
                state = "true";
            Main.confTable.put("autoconnect", state);

            String laf = (String) lafCombo.getSelectedItem();
            String theme = (String) themeCombo.getSelectedItem();
            if (laf.equals("Default")) {
                Main.confTable.put("laf", "null");

                //MetalTheme mtheme = new LightBlueTheme();
                //MetalLookAndFeel.setCurrentTheme(mtheme);

                UIDefaults table = UIManager.getDefaults();
                Font sansSerif = new Font("SansSerif", Font.PLAIN, 11);

                table.put("Label.font", sansSerif);
                table.put("SplitPane.border", BorderFactory.createEmptyBorder(
                        0, 0, 0, 0));
                table.put("Menu.font", sansSerif);
                table.put("MenuItem.font", sansSerif);
                table.put("CheckBox.font", sansSerif);
                table.put("PasswordField.font", sansSerif);
                table.put("TextField.font", sansSerif);
                table.put("TextPane.font", sansSerif);
                table.put("TextArea.font", sansSerif);
                table.put("Button.font", sansSerif);
                table.put("RadioButton.font", sansSerif);
                table.put("List.font", sansSerif);
                table.put("Statusbar.background", new Color(230, 230, 230));
                UIManager
                        .setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            } else if (laf.equals("Plastic")) {
                Main.confTable.put("laf", "PlasticLookAndFeel");
                Main.confTable.put("theme", theme);

                Class c = Class.forName("com.jgoodies.plaf.plastic.theme."
                        + theme);
                PlasticTheme ptheme = (PlasticTheme) c.newInstance();
                PlasticLookAndFeel.setMyCurrentTheme(ptheme);
                UIManager
                        .setLookAndFeel("com.jgoodies.plaf.plastic.PlasticLookAndFeel");
            } else if (laf.equals("Plastic 3D")) {
                Main.confTable.put("laf", "Plastic3DLookAndFeel");
                Main.confTable.put("theme", theme);

                Class c = Class.forName("com.jgoodies.plaf.plastic.theme."
                        + theme);
                PlasticTheme ptheme = (PlasticTheme) c.newInstance();
                PlasticLookAndFeel.setMyCurrentTheme(ptheme);
                UIManager
                        .setLookAndFeel("com.jgoodies.plaf.plastic.Plastic3DLookAndFeel");
            } else if (laf.equals("Plastic XP")) {
                Main.confTable.put("laf", "PlasticXPLookAndFeel");
                Main.confTable.put("theme", theme);

                Class c = Class.forName("com.jgoodies.plaf.plastic.theme."
                        + theme);
                PlasticTheme ptheme = (PlasticTheme) c.newInstance();
                PlasticLookAndFeel.setMyCurrentTheme(ptheme);
                UIManager
                        .setLookAndFeel("com.jgoodies.plaf.plastic.PlasticXPLookAndFeel");
            }
            SwingUtilities.updateComponentTreeUI(Main.getGUI());

            // creates a strange fault: after writing conf, values in
            // Main.confTable
            // are null
            //Main.writeConf();
        } catch (Exception ex) {
            logger.error(ex);
        }
    }
}