package de.hampelratte.yaggui.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import de.hampelratte.yaggui.Command;
import de.hampelratte.yaggui.Load;
import de.hampelratte.yaggui.Main;
import de.hampelratte.yaggui.Source;
import de.hampelratte.yaggui.Subcommand;
import de.hampelratte.yaggui.gui.sortabletable.SortableTable;
import de.hampelratte.yaggui.gui.sortabletable.SortableTableHeaderRenderer;
import de.hampelratte.yaggui.gui.sortabletable.SortableTableModel;

class UploadPanel extends JPanel implements ActionListener {
    private SortableTable table;

    private SortableTableModel model;

    private ShareTableCellRenderer stcr = new ShareTableCellRenderer();

    private JPopupMenu popup = new JPopupMenu();

    private JMenuItem popCancel = new JMenuItem("Cancel");

    UploadPanel() {
        setLayout(new BorderLayout());
        String[] columnNames = { "File", "User", "Progress", "State", "Rate",
                "Transfered/Total" };
        int[] columnWidth = { 300, 100, 100, 100, 100, 200 };

        table = new SortableTable();
        model = new SortableTableModel(6, table);
        table.setModel(model);
        model.setColumnIdentifiers(columnNames);
        table.setShowGrid(false);
        table.setRowHeight(20);
        table.setRowMargin(0);

        for (int i = model.getColumnCount() - 1; i >= 0; i--) {
            DefaultTableColumnModel tcm = (DefaultTableColumnModel) table
                    .getColumnModel();
            TableColumn tc = tcm.getColumn(i);
            if (i == 2) {
                tc.setCellRenderer(new ProgressCellRenderer());
            } else {
                tc.setCellRenderer(stcr);
            }
        }
        SortableTableHeaderRenderer renderer = new SortableTableHeaderRenderer();
        TableColumnModel model = table.getColumnModel();
        int n = columnNames.length;
        for (int i = 0; i < n; i++) {
            model.getColumn(i).setHeaderRenderer(renderer);
            model.getColumn(i).setPreferredWidth(columnWidth[i]);
        }
        JScrollPane scrollpane = new JScrollPane(table);
        scrollpane.setBorder(BorderFactory.createEmptyBorder());
        table.init();

        JLabel dirLabel = new JLabel("<html><b>Uploads</b></html>");
        //dirLabel.setBorder(BorderFactory.createEtchedBorder());
        add(dirLabel, BorderLayout.NORTH);
        add(scrollpane, BorderLayout.CENTER);

        popCancel.addActionListener(this);
        popup.add(popCancel);
        table.addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                if (e.isShiftDown() || e.isControlDown() || e.isAltDown()) {
                    return;
                }
                if (e.isMetaDown()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        });
    }

    public void addLoad(Command cmd, int session) {
        Load l = this.createLoadFromCmd(cmd, session);
        Vector row = new Vector();
        row.add(l);
        row.add(((Source) l.getSource(0)).getUser());
        row.add(new Integer(0));
        row.add(cmd.get("state"));
        row.add("0 kb/s");
        row.add("0/" + cmd.get("size"));
        model.addRow(row);
    }

    public void changeLoad(Command cmd, int session) {
        // calculate progress
        double size = new Double((String) cmd.get("size")).doubleValue();
        double transmit = new Double((String) cmd.get("transmit"))
                .doubleValue();
        int progress = (int) (transmit / size * 100);

        // calculate rate
        double throughput = new Double((String) cmd.get("throughput"))
                .doubleValue();
        double elapsed = new Double((String) cmd.get("elapsed")).doubleValue();
        double rate = (double) throughput / ((double) elapsed / 1000.0)
                / 1024.0;
        String rateString = new DecimalFormat("0.00 kb/s").format(rate);
        int bla = rateString.charAt(0);
        if (rateString.charAt(0) == 65533)
            rateString = "";

        for (int i = 0; i < model.getRowCount(); i++) {
            Load current = (Load) model.getValueAt(i, 0);
            if (current.getSession() == session) {
                model.setValueAt(new Integer(progress), i, 2);
                model.setValueAt(cmd.get("state"), i, 3);
                model.setValueAt(rateString, i, 4);
                model.setValueAt(cmd.get("transmit") + "/" + cmd.get("size"),
                        i, 5);
                repaint();
            }
        }
    }

    public void delLoad(int session) {
        for (int i = 0; i < model.getRowCount(); i++) {
            Load current = (Load) model.getValueAt(i, 0);
            if (current.getSession() == session) {
                model.removeRow(i);
            }
        }
    }

    public void actionPerformed(ActionEvent e) {
        String state = "";
        if (e.getSource() == popCancel)
            state = "cancel";

        int[] rows = table.getSelectedRows();
        for (int i = 0; i < rows.length; i++) {
            Load load = (Load) model.getValueAt(rows[i], 0);
            int session = load.getSession();
            Main.com.changeState(session, state);
        }
    }

    public void resetTable() {
        model.setRowCount(0);
    }

    private Load createLoadFromCmd(Command cmd, int session) {
        Load l = new Load(session);

        l.setFile((String) cmd.get("file"));
        l.setHash((String) cmd.get("hash"));
        l.setState((String) cmd.get("state"));
        l.setSize(Long.parseLong((String) cmd.get("size")));
        l.setTransmitted(Long.parseLong((String) cmd.get("transmit")));
        l.setShared(Boolean.getBoolean((String) cmd.get("shared")));

        String through = (String) cmd.get("throughput");
        String elapsed = (String) cmd.get("elapsed");
        if (through == null)
            through = "0";
        if (elapsed == null)
            elapsed = "0";

        l.setElapsed(Integer.parseInt(elapsed));
        l.setThroughput(Integer.parseInt(through));

        Enumeration enumeration = cmd.getSubCmds().elements();
        while (enumeration.hasMoreElements()) {
            Source src = new Source(l);
            Subcommand subcmd = (Subcommand) enumeration.nextElement();

            String user = (String) subcmd.get("user");
            String url = (String) subcmd.get("url");
            String protostate = (String) subcmd.get("status");
            String status = (String) subcmd.get("statusgrl");
            String start = (String) subcmd.get("start");
            String total = (String) subcmd.get("total");
            String transmit = (String) subcmd.get("transmit");

            if (user == null)
                user = "";
            if (url == null)
                url = "";
            if (protostate == null)
                protostate = "";
            if (status == null)
                status = "";
            if (start == null)
                start = "0";
            if (total == null)
                total = "0";
            if (transmit == null)
                transmit = "0";

            // calculate throughput
            int throughput = 0;
            try {
                Load equalOne = null;

                Vector loadsVector = new Vector();
                for (int i = 0; i < model.getRowCount(); i++) {
                    loadsVector.add(model.getValueAt(i, 0));
                }

                Enumeration enumeration2 = loadsVector.elements();
                while (enumeration2.hasMoreElements()) {
                    Load current = (Load) enumeration2.nextElement();
                    if (current.equals(l)) {
                        equalOne = current;
                    }
                }

                for (int i = 0; i < equalOne.getSourceCount(); i++) {
                    Source current = (Source) equalOne.getSource(i);
                    if (current.getUser().equals(user)) {
                        throughput = (int) (Long.parseLong(transmit) - current
                                .getTransmitted());
                    }
                }
            } catch (Exception e) {
            }

            src.setUser(user);
            src.setURL(url);
            src.setProtocolState(protostate);
            src.setStart(Long.parseLong(start));
            src.setTotal(Long.parseLong(total));
            src.setTransmitted(Long.parseLong(transmit));
            src.setThroughput(throughput);
            src.setState(status);

            l.addSource(src);
        }
        return l;
    }
}