package de.hampelratte.yaggui.gui.cddb;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;

import de.hampelratte.yaggui.cddb.FreedbReader;
import de.hampelratte.yaggui.gui.SearchPanel;

public class CDDBPanel extends JPanel implements ActionListener, MouseListener {
    private JTable table = new JTable();

    private JSplitPane splitpane = new JSplitPane();

    private JPanel statusbar = new JPanel();

    private JPanel rightPanel = new JPanel();

    private JProgressBar progress = new JProgressBar(JProgressBar.HORIZONTAL);

    private DefaultMutableTreeNode root = new DefaultMutableTreeNode(
            new CDDBResult("Results:", "", ""));

    private DefaultMutableTreeNode last;

    private JTree tree = new JTree(root);

    private JButton searchButton = new JButton("Search for selected songs");

    private SearchPanel searchpanel;

    public CDDBPanel(SearchPanel searchpanel) {
        this.searchpanel = searchpanel;
        this.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        tree.setShowsRootHandles(true);

        /*
         * DefaultTreeCellRenderer renderer =
         * (DefaultTreeCellRenderer)tree.getCellRenderer();
         * renderer.setLeafIcon(null); renderer.setClosedIcon(null);
         * renderer.setOpenIcon(null);
         */

        table.setModel(new javax.swing.table.DefaultTableModel(
                new Object[][] {}, new String[] { "Track", "Database Titles",
                        "Status" }));
        table.getColumnModel().getColumn(0).setPreferredWidth(50);
        table.getColumnModel().getColumn(0).setMaxWidth(50);
        table.getColumnModel().getColumn(1).setPreferredWidth(160);
        table.getColumnModel().getColumn(2).setPreferredWidth(50);
        table.getColumnModel().getColumn(2).setMaxWidth(50);

        rightPanel.setLayout(new BorderLayout());
        rightPanel.add(new JScrollPane(table), BorderLayout.CENTER);
        rightPanel.add(searchButton, BorderLayout.SOUTH);
        searchButton.addActionListener(this);
        searchButton.setEnabled(false);

        setLayout(new BorderLayout());
        splitpane.setLeftComponent(new JScrollPane(tree));
        splitpane.setRightComponent(rightPanel);
        splitpane.setDividerLocation(300);
        add(splitpane, BorderLayout.CENTER);

        statusbar.add(progress);
        progress.setIndeterminate(true);
        progress.setStringPainted(true);
        progress.setString("running");
        add(statusbar, BorderLayout.SOUTH);

        tree.addMouseListener(this);
    }

    public void addResult(CDDBResult result, boolean leaf) {
        if (leaf) {
            result.setTitle(((CDDBResult) last.getUserObject()).getTitle()
                    + " (" + result.getTitle() + ") ");
            last.add(new DefaultMutableTreeNode(result));
        } else {
            last = new DefaultMutableTreeNode(result);
            root.add(last);
        }
    }

    public void setCDDBDataSet(String[] result) {
        setEnded();
        for (int i = 2; i < result.length; i++) {
            Vector temp = new Vector();
            temp.add(new Integer(i - 1));
            temp.add(result[i]);
            temp.add("");
            ((DefaultTableModel) table.getModel()).addRow(temp);
        }
        searchButton.setEnabled(true);
    }

    public void setEnded() {
        progress.setIndeterminate(false);
        progress.setMaximum(100);
        progress.setValue(100);
        progress.setString("finished");
        tree.expandRow(0);
        tree.setRootVisible(false);
    }

    public void actionPerformed(ActionEvent e) {
        DefaultTableModel model = (DefaultTableModel) table.getModel();
        String[] titles = new String[table.getSelectedRowCount()];
        int[] indices = table.getSelectedRows();
        for (int i = 0; i < indices.length; i++) {

            titles[i] = (String) model.getValueAt(indices[i], 1);
            //searchpanel.addCDDBSearchTab((String)model.getValueAt(i,1));
        }
        new SearchInvoker(searchpanel, titles).start();
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2) {
            progress.setValue(0);
            progress.setIndeterminate(true);
            progress.setString("running");

            ((DefaultTableModel) table.getModel()).setRowCount(0);
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
                    .getLastSelectedPathComponent();
            CDDBResult cddb = (CDDBResult) node.getUserObject();
            FreedbReader fr = new FreedbReader(this);
            fr.getData(cddb.getCat(), cddb.getID());
        }
    }
}